#include "doubleequation.h"

DoubleEquation::DoubleEquation(double A,double B,double C)
{
    eqA = A;
    eqB = B;
    eqC = C;
}

void    DoubleEquation::Solve()
{
    if(!hasSolutons()) return;
    double D = (eqB * eqB - 4*eqA * eqC);

    if(hasRealSolutions())
    {
        realSolution1=(-eqB+sqrt(D))/(2.0 * eqA);
        realSolution2=(-eqB-sqrt(D))/(2.0 * eqA);
    }
    else
    {
        complexSolution1.setRealPart(-eqB);
        complexSolution1.setImagePart(sqrt(-D));

        complexSolution2.setRealPart(-eqB);
        complexSolution2.setImagePart(-sqrt(-D));
    }
}

void    DoubleEquation::printSolutions()
{
    if(hasRealSolutions())
    {
        printf("Roots = %lf %lf \n",realSolution1,realSolution2);
    }
    else
    {
        printf("Complex Roots\n");
        complexSolution1.printComplex();
        complexSolution2.printComplex();
    }
}

bool    DoubleEquation::hasSolutons()      const
{
    return fabs(eqA)>1e-8;
}

bool    DoubleEquation::hasRealSolutions() const
{
    return (eqB * eqB - 4*eqA * eqC)>=0;
}

bool    DoubleEquation::hasDoubleRoot()    const
{
    return fabs(eqB * eqB - 4*eqA * eqC)<=1e-8;
}

bool    DoubleEquation::hasComplexRoots()  const
{
  return !hasRealSolutions();
}

void    DoubleEquation::getRealSolutions(double &a,double &b) const
{
    if(!hasRealSolutions())
    {
        a= 1e+100;
        b= 1e+100;
    }
    else
    {
        a=realSolution1;
        b=realSolution2;
    }
}
void    DoubleEquation::getComplexSolutions(Complex &a,Complex &b) const
{
    if(hasRealSolutions())
    {
        a.setRealPart(0);
        a.setImagePart(0);
        b.setRealPart(0);
        b.setImagePart(0);
    }
    else
    {
        a = complexSolution1;
        b = complexSolution2;
    }
}

DoubleEquation::~DoubleEquation()
{

}
